@@ -1,5 +1,5 @@
 # localslackirc
-# Copyright (C) 2018-2024 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2018-2025 Salvo "LtWorf" Tomaselli
 #
 # localslackirc is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ import re
 import signal
 import socket
 import argparse
-from typing import *
+from typing import NamedTuple, Iterable
 import os
 from os import environ
 from os.path import expanduser
@@ -40,7 +40,7 @@ import msgparsing
 from diff import seddiff
 
 
-VERSION = '1.33'
+VERSION = '1.34'
 
 
 class IrcDisconnectError(Exception): ...
@@ -48,7 +48,9 @@ class IrcDisconnectError(Exception): ...
 
 class Replies(Enum):
     RPL_LUSERCLIENT = 251
+    RPL_AWAY = 301
     RPL_USERHOST = 302
+    RPL_ISON = 303
     RPL_UNAWAY = 305
     RPL_NOWAWAY = 306
     RPL_WHOISUSER = 311
@@ -71,6 +73,7 @@ class Replies(Enum):
     ERR_UNKNOWNCOMMAND = 421
     ERR_FILEERROR = 424
     ERR_ERRONEUSNICKNAME = 432
+    ERR_NEEDMOREPARAMS = 461
 
 
 class Provider(Enum):
@@ -86,12 +89,12 @@ class ClientSettings(NamedTuple):
     autojoin: bool
     no_rejoin_on_mention: bool
     provider: Provider
-    ignored_channels: Set[bytes]
-    silenced_yellers: Set[bytes]
+    ignored_channels: set[bytes]
+    silenced_yellers: set[bytes]
     downloads_directory: Path
     formatted_max_lines: int = 0
 
-    def verify(self) -> Optional[str]:
+    def verify(self) -> str | None:
         '''
         Make sure that the configuration is correct.
 
@@ -104,6 +107,10 @@ class ClientSettings(NamedTuple):
                 return f'Unable to create {self.downloads_directory}'
         elif not self.downloads_directory.is_dir():
             return f'{self.downloads_directory} is not a directory'
+
+        if not os.access(self.downloads_directory, os.W_OK | os.R_OK | os.X_OK):
+            return 'Insufficient permissions on downloads directory'
+
         return None
 
 
@@ -118,7 +125,7 @@ class Client:
         self.nick = b''
         self.username = b''
         self.realname = b''
-        self.parted_channels: Set[bytes] = settings.ignored_channels
+        self.parted_channels: set[bytes] = settings.ignored_channels
         self.known_threads: dict[bytes, slack.MessageThread] = {}
         self.hostname = gethostname().encode('utf8')
 
@@ -127,7 +134,7 @@ class Client:
         self.sl_client = sl_client
         self._usersent = False # Used to hold all events until the IRC client sends the initial USER message
         self._held_events: list[slack.SlackEvent] = []
-        self._mentions_regex_cache: dict[str, Optional[re.Pattern]] = {}  # Cache for the regexp to perform mentions. Key is channel id
+        self._mentions_regex_cache: dict[str, re.Pattern | None] = {}  # Cache for the regexp to perform mentions. Key is channel id
 
     def get_mention_str(self) -> str:
         '''
@@ -498,6 +505,38 @@ class Client:
         except Exception:
             await self._sendreply(Replies.ERR_UNKNOWNCOMMAND, f'Unable to set topic to {topic}')
 
+    async def _caphandler(self, cmd: bytes) -> None:
+        # Do absolutely nothing
+        # support for capabilities is not implemented
+        # so we just ignore it
+        return
+
+    async def _isonhandler(self, cmd: bytes) -> None:
+        users = cmd.split(b' ')
+        del users[0]
+        r = []
+
+        if len(users) < 1:
+            await self._sendreply(Replies.ERR_NEEDMOREPARAMS, 'Syntax: /ison nickname')
+            return
+
+        for username in users:
+            if b'*' in username:
+                await self._sendreply(Replies.ERR_UNKNOWNCOMMAND, 'Wildcards are not supported')
+                continue
+            uusername = username.decode()
+            try:
+                user = await self.sl_client.get_user_by_name(uusername)
+            except KeyError:
+                continue
+
+            if not await self.sl_client.isAway(user):
+                r.append(username)
+        await self._sendreply(
+            Replies.RPL_ISON,
+            b' '.join(r),
+        )
+
     async def _whoishandler(self, cmd: bytes) -> None:
         users = cmd.split(b' ')
         del users[0]
@@ -524,6 +563,8 @@ class Client:
             await self._sendreply(Replies.RPL_WHOISUSER, f'email: {user.profile.email}', [username, '', 'localhost'])
         if user.is_admin:
             await self._sendreply(Replies.RPL_WHOISOPERATOR, f'{uusername} is an IRC operator', [username])
+        if await self.sl_client.isAway(user):
+            await self._sendreply(Replies.RPL_AWAY, f'{uusername} is away', extratokens=[username])
         await self._sendreply(Replies.RPL_ENDOFWHOIS, '', extratokens=[username])
 
     async def _kickhandler(self, cmd: bytes) -> None:
@@ -622,7 +663,7 @@ class Client:
         ))
         await self.s.drain()
 
-    async def _get_regexp(self, dest: slack.User|slack.Channel) -> Optional[re.Pattern]:
+    async def _get_regexp(self, dest: slack.User|slack.Channel) -> re.Pattern | None:
         #del self._mentions_regex_cache[sl_ev.channel]
         # No nick substitutions for private chats
         if isinstance(dest, slack.User):
@@ -915,9 +956,10 @@ class Client:
             b'LISTAUTOREACT': self._listautoreacthandler,
             b'DROPAUTOREACT': self._dropautoreacthandler,
             b'QUIT': self._quithandler,
-            #CAP LS
             b'USERHOST': self._userhosthandler,
             b'WHOIS': self._whoishandler,
+            b'ISON': self._isonhandler,
+            b'CAP': self._caphandler,
         }
 
         if cmdid in handlers:
@@ -1002,7 +1044,7 @@ def main() -> None:
     openlog(environ.get('LOG_SUFFIX', args.log_suffix))
     set_debug(environ.get('DEBUG', args.debug))
 
-    status_file_str: Optional[str] = environ.get('STATUS_FILE', args.status_file)
+    status_file_str: str | None = environ.get('STATUS_FILE', args.status_file)
     status_file = None
     if status_file_str is not None:
         log('Status file at:', status_file_str)
@@ -1026,7 +1068,7 @@ def main() -> None:
     # Splitting ignored channels
     ignored_channels_str = environ.get('IGNORED_CHANNELS', args.ignored_channels)
     if autojoin and len(ignored_channels_str):
-        ignored_channels: Set[bytes] = {
+        ignored_channels: set[bytes] = {
             (b'' if i.startswith('#') else b'#') + i.encode('ascii')
             for i in ignored_channels_str.split(',')
         }
@@ -1063,7 +1105,7 @@ def main() -> None:
             exit(f'Unable to open the token file {args.tokenfile}')
 
     if 'COOKIE' in environ:
-        cookie: Optional[str] = environ['COOKIE']
+        cookie: str | None = environ['COOKIE']
     else:
         try:
             if args.cookiefile:
@@ -1084,23 +1126,17 @@ def main() -> None:
     # Parameters are dealt with
 
     async def irc_listener() -> None:
-        loop = asyncio.get_running_loop()
-
-        serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        serversocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        serversocket.bind((ip, port))
-        serversocket.listen(1)
-        serversocket.setblocking(False)
-
-        s, _ = await loop.sock_accept(serversocket)
-        serversocket.close()
-        reader, writer = await asyncio.open_connection(sock=s)
-
         previous_status = None
-        if status_file is not None and status_file.exists():
-            previous_status = status_file.read_bytes()
-        sl_client = slack.Slack(token, cookie, previous_status)
-        await sl_client.login()
+        if status_file is not None:
+            if status_file.exists():
+                if not os.access(status_file, os.O_RDWR):
+                    exit('Status file needs to have read write permissions')
+                previous_status = status_file.read_bytes()
+            else:
+                try:
+                    status_file.write_bytes(b'')
+                except Exception as e:
+                    exit(f'Failure when creating the status file {e}')
 
         clientsettings = ClientSettings(
             nouserlist=nouserlist,
@@ -1115,6 +1151,19 @@ def main() -> None:
         verify = clientsettings.verify()
         if verify is not None:
             exit(verify)
+
+        serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        serversocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        serversocket.bind((ip, port))
+        serversocket.listen(1)
+        serversocket.setblocking(False)
+        s, _ = await asyncio.get_running_loop().sock_accept(serversocket)
+        serversocket.close()
+        reader, writer = await asyncio.open_connection(sock=s)
+
+        sl_client = slack.Slack(token, cookie, previous_status)
+        await sl_client.login()
+
         ircclient = Client(writer, sl_client, clientsettings)
 
         try:
